==> rustc-1.77_1.77.2+dfsg1ubuntu1.orig.tar.xz: rustc-1.77.2-src/tests/ui/traits/vtable/vtable-multi-level.rs <==
// build-fail
#![feature(rustc_attrs)]

//   O --> G --> C --> A
//     \     \     \-> B
//     |     |-> F --> D
//     |           \-> E
//     |-> N --> J --> H
//           \     \-> I
//           |-> M --> K
//                 \-> L

#[rustc_dump_vtable]
trait A {
    //~^ error vtable
    fn foo_a(&self) {}
}

#[rustc_dump_vtable]
trait B {
    //~^ error vtable
    fn foo_b(&self) {}
}

#[rustc_dump_vtable]
trait C: A + B {
    //~^ error vtable
    fn foo_c(&self) {}
}

#[rustc_dump_vtable]
trait D {
    //~^ error vtable
    fn foo_d(&self) {}
}

#[rustc_dump_vtable]
trait E {
    //~^ error vtable
    fn foo_e(&self) {}
}

#[rustc_dump_vtable]
trait F: D + E {
    //~^ error vtable
    fn foo_f(&self) {}
}

#[rustc_dump_vtable]
trait G: C + F {
    fn foo_g(&self) {}
}

#[rustc_dump_vtable]
trait H {
    //~^ error vtable
    fn foo_h(&self) {}
}

#[rustc_dump_vtable]
trait I {
    //~^ error vtable
    fn foo_i(&self) {}
}

#[rustc_dump_vtable]
trait J: H + I {
    //~^ error vtable
    fn foo_j(&self) {}
}

#[rustc_dump_vtable]
trait K {
    //~^ error vtable
    fn foo_k(&self) {}
}

#[rustc_dump_vtable]
trait L {
    //~^ error vtable
    fn foo_l(&self) {}
}

#[rustc_dump_vtable]
trait M: K + L {
    //~^ error vtable
    fn foo_m(&self) {}
}

#[rustc_dump_vtable]
trait N: J + M {
    //~^ error vtable
    fn foo_n(&self) {}
}

#[rustc_dump_vtable]
trait O: G + N {
    //~^ error vtable
    fn foo_o(&self) {}
}

struct S;

impl A for S {}
impl B for S {}
impl C for S {}
impl D for S {}
impl E for S {}
impl F for S {}
impl G for S {}
impl H for S {}
impl I for S {}
impl J for S {}
impl K for S {}
impl L for S {}
impl M for S {}
impl N for S {}
impl O for S {}

macro_rules! monomorphize_vtable {
    ($trait:ident) => {{
        fn foo(_ : &dyn $trait) {}
        foo(&S);
    }}
}

fn main() {
    monomorphize_vtable!(O);

    monomorphize_vtable!(A);
    monomorphize_vtable!(B);
    monomorphize_vtable!(C);
    monomorphize_vtable!(D);
    monomorphize_vtable!(E);
    monomorphize_vtable!(F);
    monomorphize_vtable!(H);
    monomorphize_vtable!(I);
    monomorphize_vtable!(J);
    monomorphize_vtable!(K);
    monomorphize_vtable!(L);
    monomorphize_vtable!(M);
    monomorphize_vtable!(N);
}

==> rustc-1.89_1.89.0+dfsg0ubuntu1.orig.tar.xz: rustc-1.89.0-src/tests/ui/traits/vtable/vtable-multi-level.rs <==
#![feature(rustc_attrs)]

//   O --> G --> C --> A
//     \     \     \-> B
//     |     |-> F --> D
//     |           \-> E
//     |-> N --> J --> H
//           \     \-> I
//           |-> M --> K
//                 \-> L

trait A {
    fn foo_a(&self) {}
}

trait B {
    fn foo_b(&self) {}
}

trait C: A + B {
    fn foo_c(&self) {}
}

trait D {
    fn foo_d(&self) {}
}

trait E {
    fn foo_e(&self) {}
}

trait F: D + E {
    fn foo_f(&self) {}
}

trait G: C + F {
    fn foo_g(&self) {}
}

trait H {
    fn foo_h(&self) {}
}

trait I {
    fn foo_i(&self) {}
}

trait J: H + I {
    fn foo_j(&self) {}
}

trait K {
    fn foo_k(&self) {}
}

trait L {
    fn foo_l(&self) {}
}

trait M: K + L {
    fn foo_m(&self) {}
}

trait N: J + M {
    fn foo_n(&self) {}
}

trait O: G + N {
    fn foo_o(&self) {}
}

struct S;

#[rustc_dump_vtable]
impl A for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl B for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl C for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl D for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl E for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl F for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl G for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl H for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl I for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl J for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl K for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl L for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl M for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl N for S {}
//~^ ERROR vtable entries

#[rustc_dump_vtable]
impl O for S {}
//~^ ERROR vtable entries

fn main() {}
